MapExtension.Async = {
	Step = 0,
	MarkList = {},
	TextureList = {},
	Progress = { Mark = { Loaded = 0, Total = 0 }, Texture = {} },
	IsNeedScan = true
}

function MapExtension.Async.Init ()
	for mk, mv in pairs(MapExtension.Data.Maps) do
		MapExtension.Async.Progress.Texture[mk] = { Loaded = 0, Total = 0 }
	end
end

function MapExtension.Async.Start ()
	MapExtension.Async.Step = 1
end

function MapExtension.Async.GetProgress ()
	local progressStep = MapExtension.Async.GetProgressStep()
	if progressStep == 1 then
		return "[ Loading data... " .. math.floor(100 * MapExtension.Async.Progress.Mark.Loaded / MapExtension.Async.Progress.Mark.Total) .. "% ]"
	end
	if progressStep == 2 then
		return "[ Rendering... " .. math.floor(100 * MapExtension.Async.Progress.Texture[MapExtension.AE.UI.MapTexture.active].Loaded / MapExtension.Async.Progress.Texture[MapExtension.AE.UI.MapTexture.active].Total) .. "% ]"
	end
end

function MapExtension.Async.GetProgressStep ()
	if MapExtension.Async.Step == 1 and MapExtension.Async.Progress.Mark.Loaded < MapExtension.Async.Progress.Mark.Total then
		return 1
	end
	if MapExtension.Async.Step == 2 and MapExtension.Async.Progress.Texture[MapExtension.AE.UI.MapTexture.active].Loaded < MapExtension.Async.Progress.Texture[MapExtension.AE.UI.MapTexture.active].Total then
		return 2
	end
	if MapExtension.Async.IsNeedScan then
		MapExtension.Async.IsNeedScan = false
		MapExtension.Events.Map.Scan()
	end
	return 3
end

function MapExtension.Async.Execute ()
	if Inspect.System.Watchdog() < 0.1 then return end

	if MapExtension.Async.Step == 1 then
		MapExtension.Async.LoadMarks()
		if #MapExtension.Async.MarkList == 0 then MapExtension.Async.Step = 2 end
	end

	if MapExtension.Async.Step >= 2 then
		MapExtension.Async.LoadTextures()
		local cnt = 0
		for mk, mv in pairs(MapExtension.Data.Maps) do
			cnt = cnt + #MapExtension.Async.TextureList[mk] + #MapExtension.Async.TextureList[mk + 10]
		end
		if cnt == 0 then MapExtension.Async.Step = 3 end
	end

	local progress = MapExtension.Async.GetProgress()
	MapExtension.AE.UI.Coordinates:SetVisible(progress == nil)
	MapExtension.AE.UI.Progress:SetText(progress or "")
end

function MapExtension.Async.AddMark (category, frame, name, texture, position, data, flag, map)
	table.insert(MapExtension.Async.MarkList, { category, frame, name, texture, position, data, flag, map })
	MapExtension.Async.Progress.Mark.Total = MapExtension.Async.Progress.Mark.Total + 1
end

function MapExtension.Async.LoadMarks ()
	while Inspect.System.Watchdog() > 0.1 and #MapExtension.Async.MarkList > 0 and MapExtension.AE.Visible do
		local mark = table.remove(MapExtension.Async.MarkList, 1)
		if mark[7] == nil then mark[7] = MapExtension.Tracking.GetVisibleFlag(mark[3]) end
		MapExtension.Tracking.Marks[mark[1]] = mark[2]:AddMark(mark[3], mark[4], mark[5], mark[6], mark[7], mark[8])
		MapExtension.Async.Progress.Mark.Loaded = MapExtension.Async.Progress.Mark.Loaded + 1
	end
end

function MapExtension.Async.AddTexture (id, frame, source, texture, isIcon)
	local tid = id + iif(isIcon, 10, 0)
	if MapExtension.Async.TextureList[tid] == nil then MapExtension.Async.TextureList[tid] = {} end
	table.insert(MapExtension.Async.TextureList[tid], { frame, source, texture })
	MapExtension.Async.Progress.Texture[id].Total = MapExtension.Async.Progress.Texture[id].Total + 1
end

function MapExtension.Async.LoadTextures ()
	if #MapExtension.Async.TextureList[MapExtension.AE.UI.MapTexture.active] > 0 and MapExtension.AE.Visible then
		local texture = table.remove(MapExtension.Async.TextureList[MapExtension.AE.UI.MapTexture.active], math.floor(math.random(#MapExtension.Async.TextureList[MapExtension.AE.UI.MapTexture.active])))
		texture[1]:SetTexture(texture[2], texture[3])
		MapExtension.Async.Progress.Texture[MapExtension.AE.UI.MapTexture.active].Loaded = MapExtension.Async.Progress.Texture[MapExtension.AE.UI.MapTexture.active].Loaded + 1
	end

	local i = 0
	while Inspect.System.Watchdog() > 0.1 and #MapExtension.Async.TextureList[10 + MapExtension.AE.UI.MapTexture.active] > 0 and MapExtension.AE.Visible and i < 50 do
		local texture = table.remove(MapExtension.Async.TextureList[10 + MapExtension.AE.UI.MapTexture.active], math.floor(math.random(#MapExtension.Async.TextureList[10 + MapExtension.AE.UI.MapTexture.active])))
		texture[1]:SetTexture(texture[2], texture[3])
		texture[1]:SetVisible(texture[1].visible)
		texture[1].loaded = true
		i = i + 1
		MapExtension.Async.Progress.Texture[MapExtension.AE.UI.MapTexture.active].Loaded = MapExtension.Async.Progress.Texture[MapExtension.AE.UI.MapTexture.active].Loaded + 1
	end
end
