MapExtension.GUI.Basic = {}

function MapExtension.GUI.Basic.CreateFrame (parent, align, x, y, w, h, layer, uiParent)
	local frame = UI.CreateFrame("Frame", "", iif(uiParent == nil, parent, uiParent))

	frame:SetPoint(align, parent, "TOPLEFT", x, y)
	frame:SetWidth(w)
	frame:SetHeight(h)
	frame:SetLayer(layer)

	return frame
end

function MapExtension.GUI.Basic.CreateText (parent, align, x, y, text, layer, size, color)
	local frame = UI.CreateFrame("Text", "", parent)

	frame:SetFontSize(size or 14)
	frame:SetPoint(align, parent, align, x, y)

	if text then frame:SetText(text) end
	if layer then frame:SetLayer(layer) end
	if color then frame:SetFontColor(color[1], color[2], color[3], color[4]) end

	return frame
end

function MapExtension.GUI.Basic.CreateRiftButton (parent, x, y, layer)
	local frame = UI.CreateFrame("RiftButton", "", parent)

	frame:SetPoint("TOPRIGHT", parent, "TOPRIGHT", x, y)
	frame:SetSkin("close")
	frame:SetLayer(layer)

	return frame
end

function MapExtension.GUI.Basic.CreateButton (parent, x, y, w, h, layer, texture, layout, isSwitch)
	local frame = UI.CreateFrame("Texture", "", parent)
	if x and y then frame:SetPoint(layout or "TOPLEFT", parent, layout or "TOPLEFT", x, y) end

	frame:SetWidth(w)
	frame:SetHeight(h)
	frame:SetLayer(layer)
	frame:SetTexture(MapExtensionAddon.identifier, "img/" .. texture .. "-normal.png")
	frame.state = false

	function frame.Event:LeftDown ()
		frame:SetTexture(MapExtensionAddon.identifier, "img/" .. texture .. "-down.png")
	end

	function frame.Event:MouseIn ()
		frame:SetTexture(MapExtensionAddon.identifier, "img/" .. texture .. "-" .. iif(isSwitch and frame.state, "down", "") .. "hover.png")
	end

	function frame.Event:MouseMove ()
		frame:SetTexture(MapExtensionAddon.identifier, "img/" .. texture .. "-" .. iif(isSwitch and frame.state, "down", "") .. "hover.png")
	end

	function frame.Event:MouseOut ()
		frame:SetTexture(MapExtensionAddon.identifier, "img/" .. texture .. "-" .. iif(isSwitch and frame.state, "down", "normal") .. ".png")
	end

	function frame.SetState (flag)
		frame.state = flag
		frame:SetTexture(MapExtensionAddon.identifier, "img/" .. texture .. "-" .. iif(isSwitch and frame.state, "down", "normal") .. ".png")
	end

	return frame
end

function MapExtension.GUI.Basic.CreateCheckbox (parent, align, x, y, w, h, layer, text, callbackChange, callbackHover, param, value, iconTexture, isfolder)
	local frame = MapExtension.GUI.Basic.CreateFrame(parent, align, x, y, w, h, 0)
	frame.inner = 0

	frame.checkbox = UI.CreateFrame("RiftCheckbox", "", frame)
	frame.checkbox:SetPoint("TOPLEFT", frame, "TOPLEFT", 7, 2)
	frame.checkbox:SetLayer(layer)
	frame.checkbox:SetEnabled(true)
	frame.checkbox:SetChecked(value or false)

	frame.text = UI.CreateFrame("Text", "", frame)
	frame.text:SetPoint("TOPLEFT", frame, "TOPLEFT", iif(iconTexture == nil, 23, 45), 0)
	frame.text:SetFontSize(14)
	frame.text:SetText(text)

	if isfolder then MapExtension.GUI.Basic.CreateTexture(frame, -5, 2, 9, 15, 100, "img/folder.png", "TOPRIGHT") end
	if iconTexture ~= nil then MapExtension.GUI.Basic.CreateTexture(frame, 24, 2, 17, 17, layer):SetTexture(MapExtensionAddon.identifier, iconTexture) end

	frame.Event.LeftClick = function () frame.checkbox:SetChecked(not frame.checkbox:GetChecked()) end
        if callbackChange ~= nil then frame.checkbox.Event.CheckboxChange = function() callbackChange(frame, param) end end
	if callbackHover ~= nil then 
		frame.Event.MouseIn = function () callbackHover(frame, param) end
		frame.checkbox.Event.MouseIn = frame.Event.MouseIn
	else
		frame.Event.MouseIn = function () frame.SetHighlight(true, true) end
		frame.Event.MouseOut = function () frame.SetHighlight(false, true) end
		frame.checkbox.Event.MouseIn = frame.Event.MouseIn
		frame.checkbox.Event.MouseOut = frame.Event.MouseOut
	end
	
	function frame.SetHighlight (flag, fAuto)
		if fAuto then
			frame.inner = frame.inner + iif(flag, 1, -1)
			frame:SetBackgroundColor(1, 1, 1, iif(frame.inner > 0, .1, 0)) 
		else
			frame:SetBackgroundColor(1, 1, 1, iif(flag, .1, 0))
		end
	end

	return frame
end

function MapExtension.GUI.Basic.CreateTexture (parent, x, y, w, h, layer, texture, layout)
	local frame = UI.CreateFrame("Texture", "", parent)

	frame:SetPoint(layout or "TOPLEFT", parent, layout or "TOPLEFT", x, y)
	frame:SetWidth(w)
	frame:SetHeight(h)
	frame:SetLayer(layer)

	if texture then	frame:SetTexture(MapExtensionAddon.identifier, texture) end

	return frame
end
