MapExtension.GUI.Texture = {}

function MapExtension.GUI.Texture:Create (parent, layout, layer, texture, w, h, layoutTexture)
	local frame = UI.CreateFrame("Mask", "", parent)

	for idx, layoutItem in ipairs(layout) do
		frame:SetPoint(layoutItem.layout, parent, layoutItem.layout, layoutItem.x, layoutItem.y)
	end

	if w then frame:SetWidth(w) end
	if h then frame:SetHeight(h) end
	frame:SetLayer(layer)

	frame.texture = UI.CreateFrame("Texture", "", frame)
	frame.texture:SetTexture(MapExtensionAddon.identifier, texture)

	if layoutTexture == nil then layoutTexture = layout end
	for idx, layoutItem in ipairs(layoutTexture) do
		frame.texture:SetPoint(layoutItem.layout, frame, layoutItem.layout, 0, 0)
	end

	return frame
end

function MapExtension.GUI.Texture:CreateTile (parent, marginTop, marginRight, marginBottom, marginLeft, w, h, layer, textureC, textureH, textureV)
	self:Create(MapExtension.AE.UI.Window, {{ layout = "TOPLEFT", x = marginLeft, y = marginTop }}, layer, textureC, w, h)
	self:Create(MapExtension.AE.UI.Window, {{ layout = "TOPRIGHT", x = -marginRight, y = marginTop }}, layer, textureC, w, h)
	self:Create(MapExtension.AE.UI.Window, {{ layout = "BOTTOMLEFT", x = marginLeft, y = -marginBottom }}, layer, textureC, w, h)
	self:Create(MapExtension.AE.UI.Window, {{ layout = "BOTTOMRIGHT", x = -marginRight, y = -marginBottom }}, layer, textureC, w, h)
	self:Create(MapExtension.AE.UI.Window, {{ layout = "TOPLEFT", x = marginLeft + w, y = marginTop }, { layout = "TOPRIGHT", x = - marginRight - w, y = marginTop }}, layer, textureH, nil, h, {{ layout = "TOPLEFT", x = 0, y = 0 }, { layout = "TOPRIGHT", x = 0, y = 0 }})
	self:Create(MapExtension.AE.UI.Window, {{ layout = "BOTTOMLEFT", x = marginLeft + w, y = -marginBottom }, { layout = "BOTTOMRIGHT", x = - marginRight - w, y = -marginBottom }}, layer, textureH, nil, h, {{ layout = "BOTTOMLEFT", x = 0, y = 0 }, { layout = "BOTTOMRIGHT", x = 0, y = 0 }})
	self:Create(MapExtension.AE.UI.Window, {{ layout = "TOPLEFT", x = marginLeft, y = marginTop + h }, { layout = "BOTTOMLEFT", x = marginLeft, y = -marginBottom - h }}, layer, textureV, w, nil, {{ layout = "TOPLEFT", x = 0, y = 0 }, { layout = "BOTTOMLEFT", x = 0, y = 0 }})
	self:Create(MapExtension.AE.UI.Window, {{ layout = "TOPRIGHT", x = -marginRight, y = marginTop + h }, { layout = "BOTTOMRIGHT", x = -marginRight, y = -marginBottom - h }}, layer, textureV, w, nil, {{ layout = "TOPRIGHT", x = 0, y = 0 }, { layout = "BOTTOMRIGHT", x = 0, y = 0 }})
end

function MapExtension.GUI.Texture:CreateBlock (id, parent, texture, x, y, level)
	if level == 0 then
		self:CreateSimple(id, parent, {{ layout1 = "TOPLEFT", layout2 = "TOPLEFT" }, { layout1 = "BOTTOMRIGHT", layout2 = "CENTER" }}, texture .. string.sub("0" .. y, -2) .. "." .. string.sub("0" .. x, -2) .. ".jpg")
		self:CreateSimple(id, parent, {{ layout1 = "BOTTOMLEFT", layout2 = "CENTER" }, { layout1 = "TOPRIGHT", layout2 = "TOPRIGHT" }}, texture .. string.sub("0" .. y, -2) .. "." .. string.sub("0" .. (x + 1), -2) .. ".jpg")
		self:CreateSimple(id, parent, {{ layout1 = "BOTTOMLEFT", layout2 = "BOTTOMLEFT" }, { layout1 = "TOPRIGHT", layout2 = "CENTER" }}, texture .. string.sub("0" .. (y + 1), -2) .. "." .. string.sub("0" .. x, -2) .. ".jpg")
		self:CreateSimple(id, parent, {{ layout1 = "TOPLEFT", layout2 = "CENTER" }, { layout1 = "BOTTOMRIGHT", layout2 = "BOTTOMRIGHT" }}, texture .. string.sub("0" .. (y + 1), -2) .. "." .. string.sub("0" .. (x + 1), -2) .. ".jpg")
	else
		self:CreateBlock(id, self:CreateFrame(parent, {{ layout1 = "TOPLEFT", layout2 = "TOPLEFT" }, { layout1 = "BOTTOMRIGHT", layout2 = "CENTER" }}), texture, x, y, level - 1)
		self:CreateBlock(id, self:CreateFrame(parent, {{ layout1 = "BOTTOMLEFT", layout2 = "CENTER" }, { layout1 = "TOPRIGHT", layout2 = "TOPRIGHT" }}), texture, x + math.pow(2, level), y, level - 1)
		self:CreateBlock(id, self:CreateFrame(parent, {{ layout1 = "BOTTOMLEFT", layout2 = "BOTTOMLEFT" }, { layout1 = "TOPRIGHT", layout2 = "CENTER" }}), texture, x, y + math.pow(2, level), level - 1)
		self:CreateBlock(id, self:CreateFrame(parent, {{ layout1 = "TOPLEFT", layout2 = "CENTER" }, { layout1 = "BOTTOMRIGHT", layout2 = "BOTTOMRIGHT" }}), texture, x + math.pow(2, level), y + math.pow(2, level), level - 1)
	end
end

function MapExtension.GUI.Texture:CreateSimple (id, parent, layout, texture)
	local frame = UI.CreateFrame("Texture", "", parent)
	for idx, layoutItem in ipairs(layout) do
		frame:SetPoint(layoutItem.layout1, parent, layoutItem.layout2, iif(layoutItem.layout2 == "CENTER", iif(layoutItem.layout1 == "BOTTOMRIGHT" or layoutItem.layout1 == "TOPRIGHT", 1, 0), 0), iif(layoutItem.layout2 == "CENTER", iif(layoutItem.layout1 == "BOTTOMRIGHT" or layoutItem.layout1 == "BOTTOMLEFT", 1, 0), 0))
	end
	MapExtension.Async.AddTexture(id, frame, MapExtensionAddon.identifier, texture, false)
	return frame
end

function MapExtension.GUI.Texture:CreateFrame (parent, layout)
	local frame = UI.CreateFrame("Frame", "", parent)
	for idx, layoutItem in ipairs(layout) do
		frame:SetPoint(layoutItem.layout1, parent, layoutItem.layout2, iif(layoutItem.layout2 == "CENTER", iif(layoutItem.layout1 == "BOTTOMRIGHT" or layoutItem.layout1 == "TOPRIGHT", 1, 0), 0), iif(layoutItem.layout2 == "CENTER", iif(layoutItem.layout1 == "BOTTOMRIGHT" or layoutItem.layout1 == "BOTTOMLEFT", 1, 0), 0))
	end
	return frame
end

function MapExtension.GUI.Texture:CreateView (parent, id, marginTop, marginRight, marginBottom, marginLeft, layer, texture, width, height, borderWidth, offset)
	local frame = UI.CreateFrame("Frame", "", parent)
	frame:SetPoint("TOPLEFT", parent, "TOPLEFT", marginLeft, marginTop)
	frame:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT", -marginRight, -marginBottom)
	frame:SetLayer(layer)

	frame.zoom = 100.0
	frame.scale = 1.2
	frame.textureSize = { width = width, height = height, ratio = width / height, offset = offset }
	frame.offset = { x = 0, y = 0 }
	frame.activeOffset = { x = 0, y = 0 }
	frame.diff = { x = 0, y = 0 }

	frame.MousePosition = {}
	frame.MouseLeftClick = {}
	frame.MouseRightClick = {}
	frame.Marks = {}

	frame.mask = UI.CreateFrame("Mask", "", frame)
	frame.mask:SetAllPoints(frame)

	frame.tooltip = MapExtension.GUI.Tooltip.Init(parent)

	frame.texture = UI.CreateFrame("Frame", "", frame.mask)
	frame.texture:SetPoint("CENTER", frame.mask, "CENTER", 0, 0)
	frame.texture:SetLayer(layer)
	frame.texture:SetWidth(frame.textureSize.width)
	frame.texture:SetHeight(frame.textureSize.height)

	frame.field = UI.CreateFrame("Frame", "", frame.texture)
	frame.field:SetPoint("TOPLEFT", frame.texture, "TOPLEFT", 0, 0)
	frame.field:SetPoint("BOTTOMRIGHT", frame.texture, "BOTTOMRIGHT", 0, 0)

	self:CreateBlock(id, frame.field, texture, 0, 0, 3)

	function frame.Event:WheelForward ()
		if self.zoom > 1 then
			local mouse = Inspect.Mouse()
			self.offset.x = self.offset.x - (mouse.x - self:GetLeft() - self:GetWidth() / 2) * self.zoom * (frame.scale - 1)
			self.offset.y = self.offset.y - (mouse.y - self:GetTop() - self:GetHeight() / 2) * self.zoom * (frame.scale - 1)
			self.zoom = self.zoom / frame.scale
			self:Update()
			self:OnMousePosition()
		end
	end

	function frame.Event:WheelBack ()
		local mouse = Inspect.Mouse()
		self.offset.x = self.offset.x + (mouse.x - self:GetLeft() - self:GetWidth() / 2) * self.zoom * (frame.scale - 1)
		self.offset.y = self.offset.y + (mouse.y - self:GetTop() - self:GetHeight() / 2) * self.zoom * (frame.scale - 1)
		self.zoom = self.zoom * frame.scale
		self:Update()
		self:OnMousePosition()
	end
	
	function frame.Event:Size ()
		self:Update()
		self:OnMousePosition()
	end

	function frame.Event:LeftDown ()
		frame.mousePosition = Inspect.Mouse()
	end

	function frame.Event:LeftUp ()
		frame.offset = { x = frame.offset.x + frame.diff.x, y = frame.offset.y + frame.diff.y }
		frame.diff = { x = 0, y = 0 }
		frame.mousePosition = nil
		frame:Update()
	end

	frame.Event.LeftUpoutside = frame.Event.LeftUp

	function frame.Event:MouseMove ()
		frame.isHover = true
		if frame.mousePosition then
			local mouse = Inspect.Mouse()
			frame.diff = { x = (mouse.x - frame.mousePosition.x) * frame.zoom, 
				       y = (mouse.y - frame.mousePosition.y) * frame.zoom }
			frame:Update()
		end
		frame:OnMousePosition()
	end

	function frame.Event:MouseOut ()
		frame.isHover = false
		frame:OnMousePosition()
	end

	function frame:OnMousePosition ()
		local mouse = Inspect.Mouse()
		for idx, eventItem in ipairs(frame.MousePosition) do
			eventItem(frame.isHover, (mouse.x - frame:GetLeft() - frame:GetWidth() / 2) * frame.zoom - frame.activeOffset.x + frame.textureSize.offset.x,
						(mouse.y - frame:GetTop() - frame:GetHeight() / 2) * frame.zoom - frame.activeOffset.y + frame.textureSize.offset.y)
		end
	end

	function frame:SetPosition (x, y)
		self.offset = { x = frame.textureSize.offset.x - x, y = frame.textureSize.offset.y - y }
		self:Update()
	end

	function frame:Update ()
		self.zoom = math.min(math.max(1, self.zoom), math.max(self.textureSize.width / self:GetWidth(), self.textureSize.height / self:GetHeight()))
		if self.zoom ~= self.oldzoom then
			self.oldzoom = self.zoom
			self:UpdateMarks()
			MapExtension.GUI.Tooltip.Hide()
		end

		local mx = math.max(math.max((self.textureSize.width - self:GetWidth() * self.zoom) / 2, 0) - borderWidth, 0)
		local my = math.max(math.max((self.textureSize.height - self:GetHeight() * self.zoom) / 2, 0) - borderWidth, 0)
		self.offset = { x = math.min(math.max(self.offset.x, -mx), mx), y = math.min(math.max(self.offset.y, -my), my) }
		self.activeOffset = { x = math.min(math.max(self.offset.x + self.diff.x, -mx), mx), y = math.min(math.max(self.offset.y + self.diff.y, -my), my) }

		self.texture:SetPoint("CENTER", self.mask, "CENTER", self.activeOffset.x / self.zoom, self.activeOffset.y / self.zoom)
		self.texture:SetWidth(self.textureSize.width / self.zoom)
		self.texture:SetHeight(self.textureSize.height / self.zoom)
	end

	function frame:AddMark (name, texture, position, data, flag, map, fAsync)
		local mark = UI.CreateFrame("Texture", "", self.texture)
		mark:SetVisible(false)
		mark.position = position
		mark.data = data
		mark.visible = flag
		mark:SetLayer(position.layer or 100)

		if fAsync then
			mark:SetTexture(MapExtensionAddon.identifier, "img/marks/" .. texture .. ".png")
			mark:SetVisible(mark.visible)
			mark.loaded = true			
		else
			MapExtension.Async.AddTexture(map, mark, MapExtensionAddon.identifier, "img/marks/" .. texture .. ".png", true)
		end

		mark.Event.LeftDown = frame.Event.LeftDown
		mark.Event.LeftUp = frame.Event.LeftUp
		mark.Event.LeftUpoutside = frame.Event.LeftUp
		mark.Event.MouseMove = frame.Event.MouseMove

		function mark.Event:MouseIn () mark.hover = true mark.Show() end
		function mark.Event:MouseMove () mark.hover = true mark.Show() end
		function mark.Event:MouseOut () frame.Event:MouseOut() mark.hover = false mark.Show() end
		function mark.Show () if mark.data then if mark.hover then MapExtension.GUI.Tooltip.Show(name, mark) else MapExtension.GUI.Tooltip.Hide(name) end end end
		function mark.ChangeTexture (source, texture, width, height)
			if mark.loaded then
				if mark.textureSource == nil then 
					mark.textureSource, mark.textureName = mark:GetTexture() 
					mark.textureWidth = mark:GetWidth()
					mark.textureHeight = mark:GetHeight()
					mark.textureLayer = mark:GetLayer()
				end
				mark:SetTexture(source or mark.textureSource, texture or mark.textureName)
				mark:SetWidth(width or mark.textureWidth)
				mark:SetHeight(height or mark.textureHeight)
				mark:SetLayer(iif(source == nil, mark.textureLayer, mark.textureLayer + 100))
			end
		end

		if self.Marks[name] == nil then self.Marks[name] = {} end
		table.insert(self.Marks[name], mark)
		mark:SetPoint("CENTER", self.texture, "CENTER", (position.x - frame.textureSize.offset.x) / self.zoom, (position.y - frame.textureSize.offset.y) / self.zoom)
		return mark
	end

	function frame:SetMarkProperties (name, position, caption)
		if self.Marks[name] then
			for k, v in ipairs(self.Marks[name]) do
				v.data.caption = caption
				v.position = position
				v:SetPoint("CENTER", self.texture, "CENTER", (position.x - frame.textureSize.offset.x) / self.zoom, (position.y - frame.textureSize.offset.y) / self.zoom)
			end
		end
	end

	function frame:UpdateMarks ()
		for k, v in pairs(self.Marks) do
			for mk, mv in ipairs(v) do
				self:UpdateMark(mv)
			end
		end
	end

	function frame:SetVisibleMark (name, flag)
		if self.Marks[name] then
			for mk, mv in ipairs(self.Marks[name]) do
				if mv.loaded then
					mv:SetVisible(flag)
				end
				mv.visible = flag
				if mv.visible and mv.positioned ~= true then
					self:UpdateMark(mv)
				end
			end
		end
	end

	function frame:UpdateMark (mark)
		if mark.visible then
			mark:SetPoint("CENTER", self.texture, "CENTER", (mark.position.x - frame.textureSize.offset.x) / self.zoom, (mark.position.y - frame.textureSize.offset.y) / self.zoom)
		end
		mark.positioned = mark.visible
	end

	frame:Update()

	return frame
end
