MapExtension.Tracking = {
	ScanList = {},
	ScanListTranslate = {},
	TargetListTranslate = {},
	Marks = {},
	PointList = {},
	RareMobsList = {},
	AchievementList = {}
}

function MapExtension.Tracking.Init ()
	for dk, dv in pairs(MapExtension.Data.Tracking) do
		for k, v in ipairs(dv.Data) do
			local name = (v.displayname or v.name)[MapExtension.AE.Language] or v.name[MapExtension.AE.Language]
			MapExtension.Tracking.ScanList[v.name.English] = { name = name, category = dk, count = 0, icon = iif(v.icon == nil, nil, 'img/icons/' .. (v.icon or '')), code = dv.Code, group = dv.Group, type = dv.Type, achievement = dv.Achievement ~= nil }
			MapExtension.Tracking.ScanListTranslate[v.name[MapExtension.AE.Language]] = v.name.English
			MapExtension.Tracking.TargetListTranslate[name] = v.name.English

			if dv.Code == "raremobs" then
				MapExtension.Tracking.RareMobsList[v.name.English] = true
			end
		end

		if dv.Achievement ~= nil then
			MapExtension.Tracking.AchievementList[dk] = { count = 0, completed = 0 }
		end
	end

	for k, v in pairs(MapExtensionTrackingResources) do
		for rk, rv in pairs(v.resources) do
			if MapExtension.Tracking.ScanList[rk] ~= nil then
				MapExtension.Async.AddMark(k, MapExtension.AE.UI.MapTexture[v.map], rk, MapExtension.Tracking.ScanList[rk].code, { x = v.x, y = v.z }, { list = v.resources }, nil, v.map)
				MapExtension.Tracking.ScanList[rk].count = MapExtension.Tracking.ScanList[rk].count + 1
			end
		end
	end

	MapExtension.Menu.Tracking.UpdateCounters()
	MapExtension.Menu.Tracking.UpdateSubCounters()
end

function MapExtension.Tracking.AddElement (element, type)
	if element.description ~= nil then
		element.description = iif(type == 2, MapExtension.Tracking.TargetListTranslate[string.trim(element.description)], MapExtension.Tracking.ScanListTranslate[string.trim(element.description)])
		local zone = Inspect.Unit.Detail("player").zone

		if MapExtension.Data.Zones[zone] ~= nil and element.description ~= nil then
			zone = MapExtension.Data.Zones[zone].map

			if MapExtension.Tracking.ScanList[element.description].type == type and ((element.flags or {})[MapExtension.Tracking.ScanList[element.description].code] or element.flags == nil) then
				if type ~= 1 then
					element.coordX = math.floor((element.coordX + 5) / 10) * 10
					element.coordY = math.floor((element.coordY + 5) / 10) * 10
					element.coordZ = math.floor((element.coordZ + 5) / 10) * 10
				end

				local position = zone .. "." .. math.floor(element.coordX) .. "." .. math.floor(element.coordY) .. "." .. math.floor(element.coordZ)

				if MapExtensionTrackingResources[position] == nil then 
					MapExtensionTrackingResources[position] = { map = zone, x = math.floor(element.coordX), y = math.floor(element.coordY), z = math.floor(element.coordZ), resources = {} }
				end

				if MapExtensionTrackingResources[position].resources[element.description] ~= true then
					MapExtensionTrackingResources[position].resources[element.description] = true
					MapExtension.Tracking.ScanList[element.description].count = MapExtension.Tracking.ScanList[element.description].count + 1
					MapExtension.Menu.Tracking.UpdateSubCounters()
				end

				if MapExtension.Tracking.Marks[position] == nil then
					if MapExtension.Async.Step >= 2 then
						MapExtension.Tracking.Marks[position] = MapExtension.AE.UI.MapTexture[zone]:AddMark(element.description, MapExtension.Tracking.ScanList[element.description].code, { x = MapExtensionTrackingResources[position].x, y = MapExtensionTrackingResources[position].z }, { list = MapExtensionTrackingResources[position].resources }, MapExtension.Tracking.GetVisibleFlag(element.description), zone, true)
					else
						MapExtension.Async.AddMark(position, MapExtension.AE.UI.MapTexture[zone], element.description, MapExtension.Tracking.ScanList[element.description].code, { x = MapExtensionTrackingResources[position].x, y = MapExtensionTrackingResources[position].z }, { list = MapExtensionTrackingResources[position].resources }, nil, zone)
					end
				end

				if MapExtension.Tracking.Marks[position] ~= nil then
					MapExtension.Tracking.Marks[position].data = { list = MapExtensionTrackingResources[position].resources }
					if MapExtension.Tracking.ScanList[element.description].icon ~= nil and type == 1 then
						MapExtension.Tracking.Marks[position].ChangeTexture(MapExtensionAddon.identifier, MapExtension.Tracking.ScanList[element.description].icon, 17, 17)
					end
				end

				if type == 1 then MapExtension.Tracking.PointList[element.id] = position end
				if type == 2 then MapExtension.Tracking.CheckRare(element) end
			end
		end
	end
end

function MapExtension.Tracking.RemoveElement (id)
	if MapExtension.Tracking.PointList[id] ~= nil then
		if MapExtension.Tracking.Marks[MapExtension.Tracking.PointList[id]] ~= nil then
			MapExtension.Tracking.Marks[MapExtension.Tracking.PointList[id]].ChangeTexture()
		end

		MapExtension.Tracking.PointList[id] = nil
	end
end

function MapExtension.Tracking.ApplyFilter ()
	for dk, dv in pairs(MapExtension.Data.Tracking) do
		for k, v in ipairs(dv.Data) do
			for mk, mv in pairs(MapExtension.Data.Maps) do
				MapExtension.AE.UI.MapTexture[mk]:SetVisibleMark(v.name.English, MapExtension.Tracking.GetVisibleFlag(v.name.English))
			end
		end
	end
end

function MapExtension.Tracking.CheckRare (element)
	if MapExtension.Tracking.RareMobsList[element.description] ~= nil then
		local player = Inspect.Unit.Detail("player")
		if MapExtension.Data.Zones[player.zone] ~= nil then
			MapExtension.Messenger.Send(MapExtension.Messenger.MessageRare, { id = element.id, name = element.description, player = player.name, x = element.coordX, y = element.coordZ, zone = MapExtension.Data.Zones[player.zone].English, zoneId = player.zone, shard = MapExtension.Utils.GetCurrentShard() })
		end
	end
end

function MapExtension.Tracking.GetVisibleFlag (name)
	return ((MapExtensionUISettings.tracking[name] or not MapExtension.Tracking.ScanList[name].group) and 
		MapExtensionUISettings.tracking[MapExtension.Tracking.ScanList[name].category] and 
		(not MapExtensionUISettings.flags.filter or not MapExtension.Tracking.ScanList[name].achievement or not MapExtension.Events.Achievement.ReqList[name])) or false
end